from emod_api.demographics.Demographics import Demographics, Node
import emod_api.demographics.Demographics as Demog
from emodpy_covid.demographics.contact_mat import _get_contact_mat


import numpy as np


#********************************************************************************

age_pyr_y = {
        "AFRO:GHA": np.array([0.136, 0.125, 0.112, 0.102, 0.092, 0.082, 0.073, 0.061, 0.052, 0.045, 0.038, 0.03, 0.021, 0.013, 0.009, 0.009])
    }

def _get_age_pyr( arg_key="AFRO:GHA" ):
    """
    This function generates an age pyramid from data looked up based on geographic key.
    Args:
        Geography Key: str

    Returns:
        Age Pyramind: tuple
    """

    age_pyr_x = list(range(0,80,5))

    # confused about what this is trying to accomplish
    """
    narg_key1 = arg_key
    while(len(narg_key1) > 0 and narg_key1 not in age_pyr_y):
        narg_key1 = narg_key1.rsplit(':',1)[0]
    """

    #return(age_pyr_x, age_pyr_y[narg_key1])
    return(age_pyr_x, age_pyr_y[arg_key])


#********************************************************************************

def mat_magic( geog="AFRO:GHA", spike=False  ):
    """
    HINT matrix construction
    Args:
        geog (str): Geography Key.
        spike (bool): Contact spike
    Returns:
        HINT setup inputs : tuple
    """

    arg_dist = [1.0,1.0,1.0,1.0] # 4 1's

    # Fraction reduced work/school contacts to transfer to home
    home_acc = 0.2

    # Distribution of R0 (must be normalized)
    R0x = [   0.35,    0.50,        0.15]
    R0y = [3.0/5.0,     1.0,   29.0/15.0]

    # Age pyramid
    (age_rng, age_pyr) = _get_age_pyr( geog )

    # Unscaled contact matrices
    (mat_home, mat_work, mat_schl, mat_comm) = _get_contact_mat( geog )

    mat_home = np.array(mat_home)
    mat_work = np.array(mat_work)
    mat_schl = np.array(mat_schl)
    mat_comm = np.array(mat_comm)

    # for now let's just hardcode the hcw fraction to keep rest of code familiar
    tot_hcw_frac = 0.001
    #age_pyr_pH = np.append(age_pyr, tot_hcw_frac)
    #mat_ref_pH = np.hstack((mat_ref,col_totl_hcw[:,np.newaxis]))
    #mat_ref_pH = np.vstack((mat_ref_pH,np.append(row_totl_hcw,dia_totl_hcw)))
    #R0_ref     = np.dot(np.dot(age_pyr_pH, mat_ref_pH), age_pyr_pH)

    # Calculate HINT matrix with distancing
    mat_tot  = arg_dist[0]*mat_home + arg_dist[1]*mat_work + arg_dist[2]*mat_schl + arg_dist[3]*mat_comm

    # Move fraction from school to home and fraction from work to home
    add_fac    = home_acc*(1-arg_dist[1])
    home_mag   = add_fac*np.dot(age_pyr,mat_work)/np.dot(age_pyr,mat_home)
    mat_add1   = np.matmul(mat_home,np.diag(home_mag))

    add_fac    = home_acc*(1-arg_dist[2])
    home_mag   = add_fac*np.dot(age_pyr,mat_schl)/np.dot(age_pyr,mat_home)
    mat_add2   = np.matmul(mat_home,np.diag(home_mag))

    mat_tot    = mat_tot + mat_add1 + mat_add2

    # Increase contacts
    if(spike):
        mat_tot[:7,:7] = mat_tot[:7,:7]*np.array([[1.50,1.00,1.00,1.00,3.00,3.00,3.00],
                                                  [1.00,1.00,1.00,1.00,1.00,1.00,1.00],
                                                  [1.00,1.00,1.00,1.00,1.00,1.00,1.00],
                                                  [1.00,1.00,1.00,1.00,1.00,1.00,1.00],
                                                  [3.00,1.00,1.00,1.00,1.50,1.50,1.50],
                                                  [3.00,1.00,1.00,1.00,1.50,1.50,1.50],
                                                  [3.00,1.00,1.00,1.00,1.50,1.50,1.50]])

    # Category labels
    names_big = list()
    names_big.extend(['age{:02d}_riskLO'.format(k1) for k1 in age_rng])
    names_big.extend(['age{:02d}_riskMD'.format(k1) for k1 in age_rng])
    names_big.extend(['age{:02d}_riskHI'.format(k1) for k1 in age_rng])

    # Category fractions
    age_big = np.hstack((R0x[0]*age_pyr,  R0x[1]*age_pyr,  R0x[2]*age_pyr))

    # Tile matrix
    mat_tall = np.tile(mat_tot,(3,1))
    mat_big  = np.hstack((R0y[0]*mat_tall, R0y[1]*mat_tall, R0y[2]*mat_tall))
    #mat_big  = np.vstack((mat_big,nrowval))/R0_ref

    # Diagnostics
    #print(np.dot(np.dot(age_big,mat_big),age_big))

    return(age_big, names_big, mat_big)

 
class CovidDemographics(Demographics):
    """
    This class is derived from emod_api.demographics' Demographics class so that we can set 
    certain defaults for Covid in construction. Keen observers will note thatt SetDefaultProperties
    does not look like a Covid-specific function. But as we add other disease types the 
    generatlizations and speicfics will become clearer. The architectural point is solid.
    """
    def __init__(self, nodes, idref="Gridded world grump2.5arcmin", base_file=None, implicits=None): 
        super().__init__( nodes, idref, base_file, implicits )
        super().SetDefaultNodeAttributes()

    def fromBasicNode(lat=0, lon=0, pop=1e6, name=1, forced_id=1, implicit_config_fns = None ):
        """
        This function creates a single-node CovidDemographics instance from the params you give it. 
        """
        new_nodes = [ Node(lat=lat, lon=lon, pop=pop, name=name, forced_id=forced_id) ]
        return CovidDemographics(nodes=new_nodes, implicits=implicit_config_fns)

def from_synth_pop(tot_pop=1e6, num_nodes=100, frac_rural=0.3, id_ref="from_synth_pop", implicits=None):
    # NOTE that this is using Demographics, not CovidDemographics~
    demog = Demog.from_synth_pop( tot_pop, num_nodes, frac_rural, id_ref, implicits )  
    demog.SetDefaultNodeAttributes()
    return demog
