from emod_api import schema_to_class as s2c
import json

schema_path = None
vaccine_type = "AcquisitionBlocking"
iv_name = "SampleVaccine"
#dupe_policy = "Replace" # or "Add" or "Abort" -- from covid branch
# Note that duration (what we call waning profile) needs to be configurable, but in an intuitive way

def _SimpleMCV( timestep, coverage=1.0, mv_name=iv_name ):
    """
    MCV1 Campaign
    :param Coverage: Demographic Coverage
    :param AgeAtVaccIInDays: Delay period constant for intervention
    :param vaccine_type: Individual intervention Config
    Note Start_Day is initialized as 1, recommend that this be aligned with the start of the simulation
    """
    # First, get the objects
    event = s2c.get_class_with_defaults( "CampaignEvent", schema_path )
    coordinator = s2c.get_class_with_defaults( "StandardEventCoordinator", schema_path )
    if coordinator is None:
        print( "s2c.get_class_with_defaults returned None. Maybe no schema.json was provided." )
        return ""

    intervention = s2c.get_class_with_defaults( "SimpleVaccine", schema_path )
    efficacy_profile = "WaningEffectBox"
    waning = s2c.get_class_with_defaults( efficacy_profile, schema_path )
    waning.Initial_Effect = 0.999
    waning.Box_Duration = 365

    meta_meta_intervention = s2c.get_class_with_defaults( "DelayedIntervention", schema_path )
    meta_meta_intervention.Actual_IndividualIntervention_Configs = [ intervention ]
    meta_meta_intervention.Delay_Period_Distribution="CONSTANT_DISTRIBUTION"
    meta_meta_intervention.Delay_Period_Constant=0

    meta_intervention = s2c.get_class_with_defaults( "NodeLevelHealthTriggeredIV", schema_path )
    meta_intervention.Demographic_Coverage=coverage
    meta_intervention.Duration=-1
    meta_intervention.Trigger_Condition_List=['Births']
    meta_intervention.Actual_IndividualIntervention_Config = meta_meta_intervention

    # Second, hook them up
    event.Event_Coordinator_Config = coordinator
    coordinator.Intervention_Config = intervention
    intervention.Waning_Config = waning 
    event.Start_Day = float(timestep+1)

    # Third, do the actual settings
    intervention.Vaccine_Type = "AcquisitionBlocking"
    intervention.Intervention_Name = mv_name 
    #intervention.Duplicate_Policy = dupe_policy

    # Fourth/finally, purge the schema bits
    coordinator.finalize()
    intervention.finalize()
    waning.finalize()
    event.finalize()
    event["Name"] = "MCV1" # ???

    return event

def SimpleSample1( timestep, coverage=1.0 ):
    return _SimpleMCV( timestep, coverage, mv_name="SS1" )

def SimpleSample2( timestep, coverage=1.0 ):
    return _SimpleMCV( timestep, coverage, mv_name="SS2" )

def new_intervention_as_file( timestep, filename=None ):
    campaign = {}
    campaign["Events"] = []
    campaign["Events"].append( new_intervention( timestep, vaccine_type, iv_name ) )
    if filename is None:
        filename = "covid_vaccine.json"
    with open( filename, "w" ) as camp_file:
        json.dump( campaign, camp_file, sort_keys=True, indent=4 )
    return filename

